#include <stdio.h>
#include <math.h>


#ifdef _OPENMP
   #include <omp.h>
#else
  double ctiming();
#endif

#define NSIZE 200000
#define STEPSIZE 0.00005

int main()
{

  double xpos, ypos;
  double integral=0.0;
  int numThreads;

#pragma omp parallel default(none) shared(numThreads)
  {
    #pragma omp master
    {
     #ifdef _OPENMP
      numThreads = omp_get_num_threads();
     #else
      numThreads = 1;
     #endif
    }
  }

  /* omp_sched_t forSchedule;
  int defChunk;

  omp_get_schedule( &forSchedule, &defChunk);
  printf( "Default schedule %i, with default chunk %i\n\n", 
  forSchedule, defChunk);  */

#ifdef _OPENMP
  double starttime = omp_get_wtime();
#else
  double starttime = ctiming();
#endif

#pragma omp parallel default(none) shared(integral)
#pragma omp for private(xpos, ypos) \
  reduction(+:integral)  schedule(static,100) 


  for ( int i = 0; i < NSIZE; i++)
    {
      /*#ifdef _OPENMP
      int tid = omp_get_thread_num();
      #else
      int tid = 0;
      #endif
      //      printf("%i: I am doing %i of %i\n", tid, i, NSIZE ); */

      xpos = (i+0.5) * STEPSIZE;
      for (int j = 0; j < i; j++ )
	{
	  ypos = (j+0.5) * STEPSIZE;
	  double xvecSquare = xpos*xpos + ypos*ypos;
	  integral += exp(-xvecSquare) * STEPSIZE * STEPSIZE;
	}
    }
  
  integral *= 8.0;

#ifdef _OPENMP
  double finaltime = omp_get_wtime() - starttime;
#else
  double finaltime = ctiming() - starttime;
#endif

  printf("Approximate integral: %f for n: %i and stepsize: %f \n",
	 integral, NSIZE, STEPSIZE);

#ifdef _OPENMP  
  printf("Time taken: %f s on %i threads\n", finaltime, numThreads);
#else
  printf("Time taken: %f s for serial compile\n", finaltime);
#endif
  return 0;

}

#ifndef _OPENMP

#include <sys/time.h>



double ctiming()
{
        struct timeval tp;
        int i = gettimeofday(&tp, NULL );
        return ( (double) tp.tv_sec + (double) tp.tv_usec * 1.0e-6 );

}


#endif
